-- Circle-limits a la Escher

module CircleLimit where

import Monad
import Pan
import Simple (rotateUI,tileUI',imageToTileUI,altRingsUI)
import MiscDemos (circlify',lensPowerUI)


circleLimit :: FilterC
circleLimit im = 
  crop (disk radius) $
  im . polarXf (\ (r,a) -> (radius * r / (radius - r), a))
 where
   radius = 50

circleLimitUI :: UI FilterC
circleLimitUI = liftM (circleLimit .) scaleUI

circleLimitCheckerUI = liftM ($ cb0) circleLimitUI

-- circleLimitTileUI = composeUIs [circleLimitUI, scaleUI, tileUI]

circleLimitTileUI :: UI ImageC
circleLimitTileUI = circleLimitUI `ap` tileUI'

scaleUI = powerZoomUI "pre-scale" 7 0


-- Variation, using iteration

circleLimitTile' :: FilterC -> FloatE -> SizedImageC -> SizedImageC
circleLimitTile' preXf ang (w,h,im) =
  ( 100, 100
  , circleLimit $ preXf $ tile (w,h) $ rotate ang im
  )

-- Iterate the circle limit effect.  Strangely, the last setting is no
-- different from the second-to-last.
circleLimitTileIterUI :: Int -> UI SizedImageC -> UI ImageC
circleLimitTileIterUI lim simUI =
  do str <- scaleUI
     ang <- angleUI "rotate" (-180,180) 45
     f   <- iterUI lim (circleLimitTile' str ang)
     sim <- simUI
     let (w,h,im) = f sim
     return im


-- circleLimitRingsUI = circleLimitUI `ap` liftM toImageC altRingsUI
circleLimitRingsUI = liftM ($ ybRings) circleLimitUI


---- Move elsewhere

-- Iterate f up to lim times
iter :: Syntactic a => Int -> (a -> a) -> IntE -> (a -> a)
iter lim f n x = take lim (iterate f x) !!* n

iterUI :: Syntactic a => Int -> (a -> a) -> UI (a -> a)
iterUI lim f = liftM (iter lim f) $ islider "# iterations" (0,lim) 0

----------------------------------------

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileImageFor bends "circle limit checker" circleLimitCheckerUI
  compileImageFor bends "circle limit tiled"   circleLimitTileUI
  compileImageFor bends "circle limit tiled iterated disk" $
               circleLimitTileIterUI 3 (return (100,100,diskIm 50 red))
  compileImageFor bends "circle limit tiled iterated" $
               circleLimitTileIterUI 3 imageToTileUI
  compileImageFor bends "circle limit rings" circleLimitRingsUI
  compileImageFor bends "circle limit tiled lens" $ atMouse lensPowerUI `ap` circleLimitTileUI
